/*
 * Copyright E. Bigeon (2015)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a swt window for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc-swt:fr.bigeon.gclc.swt.SWTConsoleShellTest.java
 * Created on: Jun 8, 2016
 */
package fr.bigeon.gclc.swt;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.IOException;

import org.eclipse.swt.widgets.Display;
import org.junit.Test;

import fr.bigeon.gclc.ConsoleApplication;
import fr.bigeon.gclc.command.Command;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.InvalidCommandName;

/** <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
public class SWTConsoleShellTest {

    protected static final long TWO_SECONDS = 2000;
    private static final Display DISPLAY = new Display();

    @Test
    public void testConsoleClose() {
        final SWTConsoleShell shell = new SWTConsoleShell(DISPLAY);
        final SWTConsole swtConsole = (SWTConsole) shell.getManager();
        swtConsole.close();
        swtConsole.setPrompt(":");
        try {
            final ConsoleApplication appl = new ConsoleApplication(swtConsole,
                    "exit", "Hello", "See you");
            appl.add(new Command("long") {

                @Override
                public String tip() {
                    return "a long running command";
                }

                @Override
                public void execute(String... args) {
                    try {
                        Thread.sleep(TWO_SECONDS);
                    } catch (InterruptedException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                }
            });
            appl.add(new Command("test") {

                @Override
                public String tip() {
                    return "a prompting running command";
                }

                @Override
                public void execute(String... args) throws CommandRunException {
                    try {
                        appl.getManager().prompt("Test");
                    } catch (IOException e) {
                        throw new CommandRunException("No input", e, this);
                    }
                }
            });
//            shell.pack();
            shell.open();
            Thread applThread = new Thread(new Runnable() {

                @Override
                public void run() {
                    appl.start();
                }
            });
            Thread testThread = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        Thread.sleep(TWO_SECONDS);
                    } catch (InterruptedException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                    Display.getDefault().syncExec(new Runnable() {
                        @Override
                        public void run() {
                            swtConsole.setText("test"); //$NON-NLS-1$
                            swtConsole.validateCommand();
                        }
                    });
                    Display.getDefault().syncExec(new Runnable() {
                        @Override
                        public void run() {
                            swtConsole.setText("ok"); //$NON-NLS-1$
                        }
                    });
                    swtConsole.validateCommand();
                    Display.getDefault().syncExec(new Runnable() {
                        @Override
                        public void run() {
                            swtConsole.setText("long"); //$NON-NLS-1$
                        }
                    });
                    swtConsole.validateCommand();
                    Display.getDefault().syncExec(new Runnable() {
                        @Override
                        public void run() {
                            swtConsole.setText("test"); //$NON-NLS-1$
                        }
                    });
                    swtConsole.validateCommand();
                    Display.getDefault().syncExec(new Runnable() {
                        @Override
                        public void run() {
                            swtConsole.setText("test"); //$NON-NLS-1$
                        }
                    });
                    swtConsole.validateCommand();
                    try {
                        Thread.sleep(TWO_SECONDS);
                    } catch (InterruptedException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                    Display.getDefault().syncExec(new Runnable() {
                        @Override
                        public void run() {
                            shell.dispose();
                        }
                    });
                }
            });
            applThread.start();
            testThread.start();
            while (!shell.isDisposed()) {
                if (!DISPLAY.readAndDispatch()) {
                    DISPLAY.sleep();
                }
            }
//            DISPLAY.dispose();
            assertTrue(appl.getManager().isClosed());
            Thread.sleep(TWO_SECONDS);
            assertFalse(appl.isRunning());
        } catch (InvalidCommandName e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @Test
    public void test() {
        final SWTConsoleShell shell = new SWTConsoleShell(DISPLAY);
        final SWTConsole swtConsole = (SWTConsole) shell.getManager();
        try {
            final ConsoleApplication appl = new ConsoleApplication(swtConsole,
                    "exit", "Hello", "See you");
            appl.add(new Command("long") {

                @Override
                public String tip() {
                    return "a long running command";
                }

                @Override
                public void execute(String... args) {
                    try {
                        Thread.sleep(TWO_SECONDS);
                    } catch (InterruptedException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                }
            });
//            shell.pack();
            shell.open();
            Thread applThread = new Thread(new Runnable() {

                @Override
                public void run() {
                    appl.start();
                }
            });
            Thread testThread = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        Thread.sleep(TWO_SECONDS);
                    } catch (InterruptedException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                    Display.getDefault().syncExec(new Runnable() {
                        @Override
                        public void run() {
                            swtConsole.setText("test"); //$NON-NLS-1$
                            swtConsole.validateCommand();
                        }
                    });
                    try {
                        Thread.sleep(TWO_SECONDS);
                    } catch (InterruptedException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                    Display.getDefault().syncExec(new Runnable() {
                        @Override
                        public void run() {
                            shell.dispose();
                        }
                    });
                }
            });
            applThread.start();
            testThread.start();
            while (!shell.isDisposed()) {
                if (!DISPLAY.readAndDispatch()) {
                    DISPLAY.sleep();
                }
            }
//            DISPLAY.dispose();
            assertTrue(appl.getManager().isClosed());
            Thread.sleep(TWO_SECONDS);
            assertFalse(appl.isRunning());
        } catch (InvalidCommandName e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
}
